# -*- coding: utf-8 -*-

# system imports
import sys
import platform
from typing import Optional

# external imports
from maestral.autostart import (
    AutoStartBase,
    AutoStartLaunchd,
    AutoStartXDGDesktop,
    SupportedImplementations,
)
from maestral.constants import BUNDLE_ID


class AutoStart:
    """Creates auto-start files in the appropriate system location to automatically
    start Maestral when the user logs in. Supports launchd on macOS and XDG desktop
    entries on Linux."""

    _impl: AutoStartBase

    def __init__(self, config_name: str) -> None:
        self.implementation = self._get_available_implementation()

        start_cmd_list = [sys.executable, "-m", "maestral_qt", "-c", config_name]
        start_cmd = " ".join(start_cmd_list)
        bundle_id = "{}.{}".format(BUNDLE_ID, config_name)

        if self.implementation == SupportedImplementations.launchd:
            self._impl = AutoStartLaunchd(bundle_id, start_cmd)

        elif self.implementation == SupportedImplementations.xdg_desktop:
            additional_keys = {
                "Icon": "maestral",
                "X-GNOME-Autostart-enabled": "true",
                "X-GNOME-Autostart-Delay": "2",
                "X-KDE-autostart-after": "panel",
            }

            self._impl = AutoStartXDGDesktop(
                filename=f"maestral-{config_name}.desktop",
                app_name="Maestral",
                start_cmd=start_cmd,
                **additional_keys,
            )
        else:
            self._impl = AutoStartBase()

    @property
    def enabled(self) -> bool:
        """True if autostart is enabled."""
        return self._impl.enabled

    @enabled.setter
    def enabled(self, value: bool) -> None:
        if value:
            self.enable()
        else:
            self.disable()

    def toggle(self) -> None:
        """Toggles autostart on or off."""
        if self.enabled:
            self.disable()
        else:
            self.enable()

    def enable(self) -> None:
        """Setter: True if autostart is enabled."""
        if self.enabled:
            return
        self._impl.enable()

    def disable(self) -> None:
        """Setter: True if autostart is enabled."""
        if not self.enabled:
            return
        self._impl.disable()

    def _get_available_implementation(self) -> Optional[SupportedImplementations]:
        """Returns the supported implementation depending on the platform."""
        system = platform.system()

        if system == "Darwin":
            return SupportedImplementations.launchd
        elif system == "Linux":
            return SupportedImplementations.xdg_desktop
        else:
            return None



# Form implementation generated from reading ui file '/Users/samschott/Python/maestral-qt/maestral_qt/resources/selective_sync_dialog.ui'
#
# Created by: PyQt6 UI code generator 6.1.1
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets


class Ui_SelectiveSyncDialog(object):
    def setupUi(self, SelectiveSyncDialog):
        SelectiveSyncDialog.setObjectName("SelectiveSyncDialog")
        SelectiveSyncDialog.resize(690, 450)
        SelectiveSyncDialog.setModal(True)
        self.gridLayout = QtWidgets.QGridLayout(SelectiveSyncDialog)
        self.gridLayout.setContentsMargins(0, -1, 0, -1)
        self.gridLayout.setHorizontalSpacing(7)
        self.gridLayout.setObjectName("gridLayout")
        spacerItem = QtWidgets.QSpacerItem(10, 10, QtWidgets.QSizePolicy.Policy.Fixed, QtWidgets.QSizePolicy.Policy.Minimum)
        self.gridLayout.addItem(spacerItem, 0, 0, 2, 1)
        spacerItem1 = QtWidgets.QSpacerItem(10, 20, QtWidgets.QSizePolicy.Policy.Fixed, QtWidgets.QSizePolicy.Policy.Minimum)
        self.gridLayout.addItem(spacerItem1, 4, 4, 1, 1)
        spacerItem2 = QtWidgets.QSpacerItem(10, 20, QtWidgets.QSizePolicy.Policy.Fixed, QtWidgets.QSizePolicy.Policy.Minimum)
        self.gridLayout.addItem(spacerItem2, 4, 0, 1, 1)
        self.treeViewFolders = QtWidgets.QTreeView(SelectiveSyncDialog)
        self.treeViewFolders.setEditTriggers(QtWidgets.QAbstractItemView.EditTrigger.NoEditTriggers)
        self.treeViewFolders.setAlternatingRowColors(True)
        self.treeViewFolders.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectionBehavior.SelectRows)
        self.treeViewFolders.setTextElideMode(QtCore.Qt.TextElideMode.ElideRight)
        self.treeViewFolders.setVerticalScrollMode(QtWidgets.QAbstractItemView.ScrollMode.ScrollPerPixel)
        self.treeViewFolders.setHorizontalScrollMode(QtWidgets.QAbstractItemView.ScrollMode.ScrollPerItem)
        self.treeViewFolders.setUniformRowHeights(True)
        self.treeViewFolders.setSortingEnabled(True)
        self.treeViewFolders.setObjectName("treeViewFolders")
        self.treeViewFolders.header().setSortIndicatorShown(True)
        self.gridLayout.addWidget(self.treeViewFolders, 2, 0, 1, 5)
        spacerItem3 = QtWidgets.QSpacerItem(10, 20, QtWidgets.QSizePolicy.Policy.Fixed, QtWidgets.QSizePolicy.Policy.Minimum)
        self.gridLayout.addItem(spacerItem3, 0, 4, 2, 1)
        self.updateButton = QtWidgets.QPushButton(SelectiveSyncDialog)
        self.updateButton.setObjectName("updateButton")
        self.gridLayout.addWidget(self.updateButton, 4, 3, 1, 1)
        self.selectAllCheckBox = QtWidgets.QCheckBox(SelectiveSyncDialog)
        self.selectAllCheckBox.setObjectName("selectAllCheckBox")
        self.gridLayout.addWidget(self.selectAllCheckBox, 3, 1, 1, 1)
        self.cancelButton = QtWidgets.QPushButton(SelectiveSyncDialog)
        self.cancelButton.setObjectName("cancelButton")
        self.gridLayout.addWidget(self.cancelButton, 4, 2, 1, 1)
        spacerItem4 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.gridLayout.addItem(spacerItem4, 4, 1, 1, 1)
        self.label = QtWidgets.QLabel(SelectiveSyncDialog)
        self.label.setWordWrap(True)
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 0, 1, 1, 3)

        self.retranslateUi(SelectiveSyncDialog)
        QtCore.QMetaObject.connectSlotsByName(SelectiveSyncDialog)

    def retranslateUi(self, SelectiveSyncDialog):
        _translate = QtCore.QCoreApplication.translate
        SelectiveSyncDialog.setWindowTitle(_translate("SelectiveSyncDialog", "Dialog"))
        self.updateButton.setText(_translate("SelectiveSyncDialog", "Update"))
        self.selectAllCheckBox.setText(_translate("SelectiveSyncDialog", "Select all"))
        self.cancelButton.setText(_translate("SelectiveSyncDialog", "Cancel"))
        self.label.setText(_translate("SelectiveSyncDialog", "Select which files and folders you want to see on your computer. Any items you unselect won\'t be shown on this computer, but you can still access them on the Dropbox website."))



# -*- coding: utf-8 -*-

import os
import os.path as osp
import platform
from importlib.resources import path

from PyQt6 import QtWidgets, QtCore, QtGui

_icon_provider = QtWidgets.QFileIconProvider()
_tmp_file_for_ext = dict()


def resource_path(name):
    """Returns the resource path as a string. Extracts the resource if necessary."""
    return str(path("maestral_qt.resources", name).__enter__())


APP_ICON_PATH = resource_path("maestral.png")
FACEHOLDER_PATH = resource_path("faceholder.png")

THEME_DARK = "dark"
THEME_LIGHT = "light"


def _get_desktop():
    """
    Determines the current desktop environment. This is used for instance to decide
    which keyring backend is preferred to store the auth token.

    :returns: 'gnome', 'kde', 'xfce', 'cocoa', '' or any other string if the desktop
        $XDG_CURRENT_DESKTOP if the desktop environment is not known to us.
    :rtype: str
    """

    if platform.system() == "Linux":
        current_desktop = os.environ.get("XDG_CURRENT_DESKTOP", "").lower()
        desktop_session = os.environ.get("GDMSESSION", "").lower()

        for desktop in ("gnome", "kde", "xfce", "mate"):
            if desktop in current_desktop or desktop in desktop_session:
                return desktop

        return "unknown"

    elif platform.system() == "Darwin":
        return "cocoa"


DESKTOP = _get_desktop()


def native_item_icon(item_path):
    """Returns the system icon for the given file or folder. If there is no item at the
    given path, the systems default file icon will be returned.

    :param str item_path: Path to local item.
    """
    while not osp.exists(item_path):
        # We create a temporary file with the given extension to get the correct icon
        # this is necessary because QFileIconProvider has no API to get an icon for a
        # file extension.
        _, extension = osp.splitext(item_path)
        try:
            item_path = _tmp_file_for_ext[extension]
        except KeyError:
            tmp_file = QtCore.QTemporaryFile(
                osp.join(QtCore.QDir.tempPath(), f"XXXXXX{extension}")
            )
            tmp_file.setAutoRemove(False)
            # open and close to create
            tmp_file.open()
            tmp_file.close()

            item_path = tmp_file.fileName()
            _tmp_file_for_ext[extension] = item_path

    return _icon_provider.icon(QtCore.QFileInfo(item_path))


def native_folder_icon():
    """Returns the system's default folder icon."""
    # use a real folder here because Qt may otherwise
    # return the wrong folder icon in some cases
    return _icon_provider.icon(QtCore.QFileInfo("/usr"))


def native_file_icon():
    """Returns the system's default file icon."""
    # use a real file here because Qt may otherwise
    # return the wrong folder icon in some cases
    return _icon_provider.icon(QtCore.QFileInfo(resource_path("file")))


# noinspection PyCallByClass,PyArgumentList
def system_tray_icon(status, geometry=None):
    """Returns the system tray icon for the given status. The following icons
    will be used:

    1) macOS: Black SVG icons with transparent background. macOS will adapt the
       appearance as necessary. This only works reliably in Qt >= 5.15.
    2) Linux: Any icons installed in the system theme. Maestral will prefer "symbolic"
       icons named "maestral-status-{status}-symbolic" over regular icons named
       "maestral-status-{status}". It will fall back to manually setting the icon in a
       color which contrasts the background color of the status bar.

    :param str status: Maestral status. Must be 'idle', 'syncing', 'paused',
        'disconnected' 'info' or 'error'.
    :param geometry: Tray icon geometry on screen. If given, this location will be used
        to determine the system tray background color.
    """
    allowed_status = {"idle", "syncing", "paused", "disconnected", "info", "error"}
    if status not in allowed_status:
        raise ValueError(f"status must be in {allowed_status}")

    if platform.system() == "Darwin":
        # use SVG icon with automatic color
        icon = QtGui.QIcon(resource_path(f"maestral_tray-{status}-dark.svg"))
        icon.setIsMask(True)

    else:
        # Prefer icons from theme if installed / existing. Fall back to loading our own
        # SVG icon with a color contrasting the status bar.

        theme_icon_name = f"maestral_tray-{status}"
        theme_icon_name_symbolic = f"maestral_tray-{status}-symbolic"

        # Prefer "symbolic" icons where the appearance is adapted by the platform
        # automatically. Specs for symbolic icons and their use in the system tray
        # vary between platforms. Fall back to regular icons instead.

        if QtGui.QIcon.hasThemeIcon(theme_icon_name_symbolic):
            icon = QtGui.QIcon.fromTheme(theme_icon_name_symbolic)
        elif QtGui.QIcon.hasThemeIcon(theme_icon_name):
            icon = QtGui.QIcon.fromTheme(theme_icon_name)
        else:
            icon = QtGui.QIcon()

        if icon.isNull():
            color = "light" if is_dark_status_bar(geometry) else "dark"

            # We create our icon from a pixmap instead of the SVG directly, this works
            # around https://bugreports.qt.io/browse/QTBUG-53550.
            pixmap = QtGui.QPixmap(resource_path(f"maestral_tray-{status}-{color}.svg"))
            icon = QtGui.QIcon(pixmap)

    return icon


# noinspection PyArgumentList
def systray_theme(icon_geometry=None):
    """
    Returns one of THEME_LIGHT or THEME_DARK, corresponding to the current status bar
    color.

    ``icon_geometry`` provides the geometry (location and dimensions) of the tray icon.
    If not given, we try to guess the location of the system tray.
    """

    # ---- check for the status bar color ----------------------------------------------

    # see if we can trust returned pixel colors
    # (work around for a bug in Qt with KDE where all screenshots return black)

    c0 = _pixel_at(10, 10)
    c1 = _pixel_at(300, 400)
    c2 = _pixel_at(800, 800)

    if not c0 == c1 == c2 == (0, 0, 0):  # we can trust pixel colors from screenshots
        if not icon_geometry or icon_geometry.isEmpty():
            # ---- guess the location of the status bar --------------------------------

            primary_screen = QtGui.QGuiApplication.primaryScreen()

            rec_screen = primary_screen.geometry()
            rec_available = primary_screen.availableGeometry()

            # convert to QRegion for subtraction
            region_screen = QtGui.QRegion(rec_screen)
            region_available = QtGui.QRegion(rec_available)

            # subtract and convert back to QRect
            task_bar_rect = region_screen.subtracted(region_available).boundingRect()

            px = task_bar_rect.left() + 2
            py = task_bar_rect.bottom() - 2

        else:  # use the given location from icon_geometry
            px = icon_geometry.left()
            py = icon_geometry.bottom()

        # get pixel luminance from icon corner or status bar
        pixel_rgb = _pixel_at(px, py)
        lum = rgb_to_luminance(*pixel_rgb)

        return THEME_LIGHT if lum >= 0.4 else THEME_DARK

    else:
        # ---- give up, default to dark ------------------------------------------------
        return THEME_DARK


def is_dark_status_bar(icon_geometry=None):
    """Detects the current status bar brightness and returns ``True`` for a dark status
    bar. ``icon_geometry`` provides the geometry (location and dimensions) of the tray
    icon. If not given, we try to guess the location of the system tray."""
    return systray_theme(icon_geometry) == THEME_DARK


def rgb_to_luminance(r, g, b, base=256):
    """
    Calculates luminance of a color, on a scale from 0 to 1, meaning that 1 is the
    highest luminance. r, g, b arguments values should be in 0..256 limits, or base
    argument should define the upper limit otherwise.
    """
    return (0.2126 * r + 0.7152 * g + 0.0722 * b) / base


# noinspection PyArgumentList
def _pixel_at(x, y):
    """
    Returns (r, g, b) color code for a pixel with given coordinates (each value is in
    0..256 limits)
    """

    screen = QtGui.QGuiApplication.primaryScreen()
    color = screen.grabWindow(0, x, y, 1, 1).toImage().pixel(0, 0)

    return ((color >> 16) & 0xFF), ((color >> 8) & 0xFF), (color & 0xFF)



# Form implementation generated from reading ui file '/Users/samschott/Python/maestral-qt/maestral_qt/resources/relink_dialog.ui'
#
# Created by: PyQt6 UI code generator 6.1.1
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets


class Ui_RelinkDialog(object):
    def setupUi(self, RelinkDialog):
        RelinkDialog.setObjectName("RelinkDialog")
        RelinkDialog.resize(470, 157)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Preferred, QtWidgets.QSizePolicy.Policy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(RelinkDialog.sizePolicy().hasHeightForWidth())
        RelinkDialog.setSizePolicy(sizePolicy)
        RelinkDialog.setMinimumSize(QtCore.QSize(470, 0))
        RelinkDialog.setMaximumSize(QtCore.QSize(470, 16777215))
        RelinkDialog.setWindowTitle("")
        self.gridLayout = QtWidgets.QGridLayout(RelinkDialog)
        self.gridLayout.setHorizontalSpacing(15)
        self.gridLayout.setObjectName("gridLayout")
        self.infoLabel = QtWidgets.QLabel(RelinkDialog)
        self.infoLabel.setMinimumSize(QtCore.QSize(0, 50))
        self.infoLabel.setWordWrap(True)
        self.infoLabel.setOpenExternalLinks(True)
        self.infoLabel.setObjectName("infoLabel")
        self.gridLayout.addWidget(self.infoLabel, 1, 1, 1, 6)
        self.iconLabel = QtWidgets.QLabel(RelinkDialog)
        self.iconLabel.setMinimumSize(QtCore.QSize(60, 60))
        self.iconLabel.setMaximumSize(QtCore.QSize(60, 60))
        self.iconLabel.setText("")
        self.iconLabel.setObjectName("iconLabel")
        self.gridLayout.addWidget(self.iconLabel, 0, 0, 2, 1, QtCore.Qt.AlignmentFlag.AlignTop)
        self.pushButtonUnlink = QtWidgets.QPushButton(RelinkDialog)
        self.pushButtonUnlink.setObjectName("pushButtonUnlink")
        self.gridLayout.addWidget(self.pushButtonUnlink, 3, 3, 1, 1)
        self.pushButtonLink = QtWidgets.QPushButton(RelinkDialog)
        self.pushButtonLink.setEnabled(False)
        self.pushButtonLink.setObjectName("pushButtonLink")
        self.gridLayout.addWidget(self.pushButtonLink, 3, 6, 1, 1)
        self.titleLabel = QtWidgets.QLabel(RelinkDialog)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.titleLabel.setFont(font)
        self.titleLabel.setObjectName("titleLabel")
        self.gridLayout.addWidget(self.titleLabel, 0, 1, 1, 6)
        self.lineEditAuthCode = QtWidgets.QLineEdit(RelinkDialog)
        self.lineEditAuthCode.setText("")
        self.lineEditAuthCode.setAlignment(QtCore.Qt.AlignmentFlag.AlignLeading|QtCore.Qt.AlignmentFlag.AlignLeft|QtCore.Qt.AlignmentFlag.AlignVCenter)
        self.lineEditAuthCode.setObjectName("lineEditAuthCode")
        self.gridLayout.addWidget(self.lineEditAuthCode, 2, 1, 1, 6)
        self.pushButtonCancel = QtWidgets.QPushButton(RelinkDialog)
        self.pushButtonCancel.setObjectName("pushButtonCancel")
        self.gridLayout.addWidget(self.pushButtonCancel, 3, 5, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(1, 20, QtWidgets.QSizePolicy.Policy.Fixed, QtWidgets.QSizePolicy.Policy.Minimum)
        self.gridLayout.addItem(spacerItem, 3, 4, 1, 1)
        self.progressIndicator = QProgressIndicator(RelinkDialog)
        self.progressIndicator.setMinimumSize(QtCore.QSize(16, 16))
        self.progressIndicator.setMaximumSize(QtCore.QSize(16, 16))
        self.progressIndicator.setObjectName("progressIndicator")
        self.gridLayout.addWidget(self.progressIndicator, 3, 1, 1, 1, QtCore.Qt.AlignmentFlag.AlignVCenter)
        self.gridLayout.setRowStretch(1, 1)
        self.gridLayout.setRowStretch(2, 1)

        self.retranslateUi(RelinkDialog)
        QtCore.QMetaObject.connectSlotsByName(RelinkDialog)
        RelinkDialog.setTabOrder(self.lineEditAuthCode, self.pushButtonLink)
        RelinkDialog.setTabOrder(self.pushButtonLink, self.pushButtonCancel)
        RelinkDialog.setTabOrder(self.pushButtonCancel, self.pushButtonUnlink)

    def retranslateUi(self, RelinkDialog):
        _translate = QtCore.QCoreApplication.translate
        self.infoLabel.setText(_translate("RelinkDialog", "Your Dropbox access {0}. To continue syncing, please click <a href=\"{1}\">here</a> to retrieve a new authorization token from Dropbox and enter it below."))
        self.pushButtonUnlink.setText(_translate("RelinkDialog", "Unlink and Quit"))
        self.pushButtonLink.setText(_translate("RelinkDialog", "Link"))
        self.titleLabel.setText(_translate("RelinkDialog", "Expired Dropbox access"))
        self.lineEditAuthCode.setPlaceholderText(_translate("RelinkDialog", "Authorization token"))
        self.pushButtonCancel.setText(_translate("RelinkDialog", "Quit"))
from maestral_qt.widgets import QProgressIndicator



# Form implementation generated from reading ui file 'settings_window.ui'
#
# Created by: PyQt6 UI code generator 6.4.1
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets


class Ui_SettingsWindow(object):
    def setupUi(self, SettingsWindow):
        SettingsWindow.setObjectName("SettingsWindow")
        SettingsWindow.resize(600, 457)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Fixed, QtWidgets.QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(SettingsWindow.sizePolicy().hasHeightForWidth())
        SettingsWindow.setSizePolicy(sizePolicy)
        SettingsWindow.setMinimumSize(QtCore.QSize(600, 0))
        self.verticalLayout = QtWidgets.QVBoxLayout(SettingsWindow)
        self.verticalLayout.setContentsMargins(20, 20, 20, 20)
        self.verticalLayout.setObjectName("verticalLayout")
        spacerItem = QtWidgets.QSpacerItem(20, 0, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.verticalLayout.addItem(spacerItem)
        self.gridLayoutMain = QtWidgets.QGridLayout()
        self.gridLayoutMain.setObjectName("gridLayoutMain")
        self.comboBoxUpdateInterval = CustomCombobox(parent=SettingsWindow)
        self.comboBoxUpdateInterval.setObjectName("comboBoxUpdateInterval")
        self.comboBoxUpdateInterval.addItem("")
        self.comboBoxUpdateInterval.addItem("")
        self.comboBoxUpdateInterval.addItem("")
        self.comboBoxUpdateInterval.addItem("")
        self.gridLayoutMain.addWidget(self.comboBoxUpdateInterval, 8, 1, 1, 1)
        self.labelExcludedFoldersTitle = QtWidgets.QLabel(parent=SettingsWindow)
        self.labelExcludedFoldersTitle.setAlignment(QtCore.Qt.AlignmentFlag.AlignRight|QtCore.Qt.AlignmentFlag.AlignTrailing|QtCore.Qt.AlignmentFlag.AlignVCenter)
        self.labelExcludedFoldersTitle.setObjectName("labelExcludedFoldersTitle")
        self.gridLayoutMain.addWidget(self.labelExcludedFoldersTitle, 4, 0, 1, 1)
        self.widgetCLI = QtWidgets.QWidget(parent=SettingsWindow)
        self.widgetCLI.setObjectName("widgetCLI")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.widgetCLI)
        self.verticalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.gridLayoutMain.addWidget(self.widgetCLI, 11, 0, 1, 3)
        self.line2 = QtWidgets.QFrame(parent=SettingsWindow)
        self.line2.setStyleSheet("color: rgb(213, 213, 213)")
        self.line2.setFrameShadow(QtWidgets.QFrame.Shadow.Plain)
        self.line2.setFrameShape(QtWidgets.QFrame.Shape.HLine)
        self.line2.setObjectName("line2")
        self.gridLayoutMain.addWidget(self.line2, 7, 0, 1, 3)
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setHorizontalSpacing(0)
        self.gridLayout.setVerticalSpacing(5)
        self.gridLayout.setObjectName("gridLayout")
        self.labelUrl = QtWidgets.QLabel(parent=SettingsWindow)
        self.labelUrl.setOpenExternalLinks(True)
        self.labelUrl.setObjectName("labelUrl")
        self.gridLayout.addWidget(self.labelUrl, 1, 0, 1, 1)
        self.labelVersion = QtWidgets.QLabel(parent=SettingsWindow)
        self.labelVersion.setTextFormat(QtCore.Qt.TextFormat.PlainText)
        self.labelVersion.setAlignment(QtCore.Qt.AlignmentFlag.AlignLeading|QtCore.Qt.AlignmentFlag.AlignLeft|QtCore.Qt.AlignmentFlag.AlignVCenter)
        self.labelVersion.setObjectName("labelVersion")
        self.gridLayout.addWidget(self.labelVersion, 0, 0, 1, 1)
        self.labelCopyright = QtWidgets.QLabel(parent=SettingsWindow)
        self.labelCopyright.setStyleSheet("color: rgb(127, 127, 127)")
        self.labelCopyright.setObjectName("labelCopyright")
        self.gridLayout.addWidget(self.labelCopyright, 2, 0, 1, 1)
        self.gridLayoutMain.addLayout(self.gridLayout, 13, 1, 1, 2)
        self.comboBoxDropboxPath = CustomCombobox(parent=SettingsWindow)
        self.comboBoxDropboxPath.setObjectName("comboBoxDropboxPath")
        self.gridLayoutMain.addWidget(self.comboBoxDropboxPath, 5, 1, 1, 1)
        self.line1 = QtWidgets.QFrame(parent=SettingsWindow)
        self.line1.setMinimumSize(QtCore.QSize(200, 0))
        self.line1.setStyleSheet("color: rgb(213, 213, 213)")
        self.line1.setFrameShadow(QtWidgets.QFrame.Shadow.Plain)
        self.line1.setFrameShape(QtWidgets.QFrame.Shape.HLine)
        self.line1.setObjectName("line1")
        self.gridLayoutMain.addWidget(self.line1, 3, 0, 1, 3)
        self.line0 = QtWidgets.QFrame(parent=SettingsWindow)
        self.line0.setStyleSheet("color: rgb(213, 213, 213)")
        self.line0.setFrameShadow(QtWidgets.QFrame.Shadow.Plain)
        self.line0.setFrameShape(QtWidgets.QFrame.Shape.HLine)
        self.line0.setObjectName("line0")
        self.gridLayoutMain.addWidget(self.line0, 12, 0, 1, 3)
        self.labelAboutTitle = QtWidgets.QLabel(parent=SettingsWindow)
        self.labelAboutTitle.setAlignment(QtCore.Qt.AlignmentFlag.AlignRight|QtCore.Qt.AlignmentFlag.AlignTop|QtCore.Qt.AlignmentFlag.AlignTrailing)
        self.labelAboutTitle.setObjectName("labelAboutTitle")
        self.gridLayoutMain.addWidget(self.labelAboutTitle, 13, 0, 1, 1)
        self.checkBoxNotifications = QtWidgets.QCheckBox(parent=SettingsWindow)
        self.checkBoxNotifications.setChecked(True)
        self.checkBoxNotifications.setObjectName("checkBoxNotifications")
        self.gridLayoutMain.addWidget(self.checkBoxNotifications, 10, 1, 1, 1)
        self.pushButtonUnlink = QtWidgets.QPushButton(parent=SettingsWindow)
        self.pushButtonUnlink.setObjectName("pushButtonUnlink")
        self.gridLayoutMain.addWidget(self.pushButtonUnlink, 2, 1, 1, 1)
        self.labelDropboxPathTitle = QtWidgets.QLabel(parent=SettingsWindow)
        self.labelDropboxPathTitle.setAlignment(QtCore.Qt.AlignmentFlag.AlignRight|QtCore.Qt.AlignmentFlag.AlignTrailing|QtCore.Qt.AlignmentFlag.AlignVCenter)
        self.labelDropboxPathTitle.setObjectName("labelDropboxPathTitle")
        self.gridLayoutMain.addWidget(self.labelDropboxPathTitle, 5, 0, 1, 1)
        self.verticalLayoutAccountInfo = QtWidgets.QVBoxLayout()
        self.verticalLayoutAccountInfo.setSpacing(2)
        self.verticalLayoutAccountInfo.setObjectName("verticalLayoutAccountInfo")
        self.labelAccountInfo = QtWidgets.QLabel(parent=SettingsWindow)
        font = QtGui.QFont()
        font.setPointSize(12)
        self.labelAccountInfo.setFont(font)
        self.labelAccountInfo.setObjectName("labelAccountInfo")
        self.verticalLayoutAccountInfo.addWidget(self.labelAccountInfo)
        self.labelSpaceUsage = QtWidgets.QLabel(parent=SettingsWindow)
        font = QtGui.QFont()
        font.setPointSize(12)
        self.labelSpaceUsage.setFont(font)
        self.labelSpaceUsage.setObjectName("labelSpaceUsage")
        self.verticalLayoutAccountInfo.addWidget(self.labelSpaceUsage)
        self.gridLayoutMain.addLayout(self.verticalLayoutAccountInfo, 1, 1, 1, 2)
        self.labelUpdateInterval = QtWidgets.QLabel(parent=SettingsWindow)
        self.labelUpdateInterval.setAlignment(QtCore.Qt.AlignmentFlag.AlignRight|QtCore.Qt.AlignmentFlag.AlignTrailing|QtCore.Qt.AlignmentFlag.AlignVCenter)
        self.labelUpdateInterval.setObjectName("labelUpdateInterval")
        self.gridLayoutMain.addWidget(self.labelUpdateInterval, 8, 0, 1, 1)
        self.labelUserProfilePic = QtWidgets.QLabel(parent=SettingsWindow)
        self.labelUserProfilePic.setText("")
        self.labelUserProfilePic.setAlignment(QtCore.Qt.AlignmentFlag.AlignRight|QtCore.Qt.AlignmentFlag.AlignTop|QtCore.Qt.AlignmentFlag.AlignTrailing)
        self.labelUserProfilePic.setObjectName("labelUserProfilePic")
        self.gridLayoutMain.addWidget(self.labelUserProfilePic, 0, 0, 3, 1, QtCore.Qt.AlignmentFlag.AlignRight)
        self.pushButtonExcludedFolders = QtWidgets.QPushButton(parent=SettingsWindow)
        self.pushButtonExcludedFolders.setObjectName("pushButtonExcludedFolders")
        self.gridLayoutMain.addWidget(self.pushButtonExcludedFolders, 4, 1, 1, 1)
        self.checkBoxStartup = QtWidgets.QCheckBox(parent=SettingsWindow)
        self.checkBoxStartup.setChecked(True)
        self.checkBoxStartup.setObjectName("checkBoxStartup")
        self.gridLayoutMain.addWidget(self.checkBoxStartup, 9, 1, 1, 1)
        self.labelAccountName = QtWidgets.QLabel(parent=SettingsWindow)
        font = QtGui.QFont()
        font.setPointSize(20)
        self.labelAccountName.setFont(font)
        self.labelAccountName.setAlignment(QtCore.Qt.AlignmentFlag.AlignBottom|QtCore.Qt.AlignmentFlag.AlignLeading|QtCore.Qt.AlignmentFlag.AlignLeft)
        self.labelAccountName.setObjectName("labelAccountName")
        self.gridLayoutMain.addWidget(self.labelAccountName, 0, 1, 1, 2)
        self.labelSystemSettings = QtWidgets.QLabel(parent=SettingsWindow)
        self.labelSystemSettings.setAlignment(QtCore.Qt.AlignmentFlag.AlignRight|QtCore.Qt.AlignmentFlag.AlignTrailing|QtCore.Qt.AlignmentFlag.AlignVCenter)
        self.labelSystemSettings.setObjectName("labelSystemSettings")
        self.gridLayoutMain.addWidget(self.labelSystemSettings, 9, 0, 1, 1)
        self.labelBandwidthLimits = QtWidgets.QLabel(parent=SettingsWindow)
        self.labelBandwidthLimits.setAlignment(QtCore.Qt.AlignmentFlag.AlignRight|QtCore.Qt.AlignmentFlag.AlignTrailing|QtCore.Qt.AlignmentFlag.AlignVCenter)
        self.labelBandwidthLimits.setObjectName("labelBandwidthLimits")
        self.gridLayoutMain.addWidget(self.labelBandwidthLimits, 6, 0, 1, 1)
        self.pushButtonBandwidthLimits = QtWidgets.QPushButton(parent=SettingsWindow)
        self.pushButtonBandwidthLimits.setObjectName("pushButtonBandwidthLimits")
        self.gridLayoutMain.addWidget(self.pushButtonBandwidthLimits, 6, 1, 1, 1)
        self.gridLayoutMain.setColumnStretch(0, 4)
        self.gridLayoutMain.setColumnStretch(1, 4)
        self.gridLayoutMain.setColumnStretch(2, 3)
        self.verticalLayout.addLayout(self.gridLayoutMain)
        spacerItem1 = QtWidgets.QSpacerItem(20, 0, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.verticalLayout.addItem(spacerItem1)

        self.retranslateUi(SettingsWindow)
        QtCore.QMetaObject.connectSlotsByName(SettingsWindow)
        SettingsWindow.setTabOrder(self.pushButtonUnlink, self.pushButtonExcludedFolders)
        SettingsWindow.setTabOrder(self.pushButtonExcludedFolders, self.comboBoxDropboxPath)
        SettingsWindow.setTabOrder(self.comboBoxDropboxPath, self.comboBoxUpdateInterval)
        SettingsWindow.setTabOrder(self.comboBoxUpdateInterval, self.checkBoxStartup)
        SettingsWindow.setTabOrder(self.checkBoxStartup, self.checkBoxNotifications)

    def retranslateUi(self, SettingsWindow):
        _translate = QtCore.QCoreApplication.translate
        SettingsWindow.setWindowTitle(_translate("SettingsWindow", "Maestral Settings"))
        self.comboBoxUpdateInterval.setItemText(0, _translate("SettingsWindow", "Daily"))
        self.comboBoxUpdateInterval.setItemText(1, _translate("SettingsWindow", "Weekly"))
        self.comboBoxUpdateInterval.setItemText(2, _translate("SettingsWindow", "Monthly"))
        self.comboBoxUpdateInterval.setItemText(3, _translate("SettingsWindow", "Never"))
        self.labelExcludedFoldersTitle.setText(_translate("SettingsWindow", "Selective sync:"))
        self.labelUrl.setText(_translate("SettingsWindow", "<html><head/><body><p><a href=\"{0}\">{0}</span></a></p></body></html>"))
        self.labelVersion.setText(_translate("SettingsWindow", "GUI v{0}, daemon v{1}"))
        self.labelCopyright.setText(_translate("SettingsWindow", "© 2018-{0}, {1}."))
        self.labelAboutTitle.setText(_translate("SettingsWindow", "About Maestral:"))
        self.checkBoxNotifications.setText(_translate("SettingsWindow", "Enable notifications on file changes"))
        self.pushButtonUnlink.setText(_translate("SettingsWindow", "Unlink this Dropbox..."))
        self.labelDropboxPathTitle.setText(_translate("SettingsWindow", "Local Dropbox folder:"))
        self.labelAccountInfo.setText(_translate("SettingsWindow", "mail@outlook.com, Dropbox Business"))
        self.labelSpaceUsage.setText(_translate("SettingsWindow", "31.8% of 1,324,980GB used"))
        self.labelUpdateInterval.setText(_translate("SettingsWindow", "Check for updates:"))
        self.pushButtonExcludedFolders.setText(_translate("SettingsWindow", "Select files and folders..."))
        self.checkBoxStartup.setText(_translate("SettingsWindow", "Start Maestral on login"))
        self.labelAccountName.setText(_translate("SettingsWindow", "User Name"))
        self.labelSystemSettings.setText(_translate("SettingsWindow", "System settings:"))
        self.labelBandwidthLimits.setText(_translate("SettingsWindow", "Bandwidth limits:"))
        self.pushButtonBandwidthLimits.setText(_translate("SettingsWindow", "Change settings..."))
from maestral_qt.widgets import CustomCombobox



# -*- coding: utf-8 -*-

# system imports
import os.path as osp
from urllib import parse
from datetime import datetime

# external packages
import click
from PyQt6 import QtCore, QtGui, QtWidgets
from maestral.models import ItemType
from maestral.utils import sanitize_string

# local imports
from .resources import (
    native_item_icon,
    native_folder_icon,
)
from .utils import (
    icon_to_pixmap,
    center_window,
    get_scaled_font,
    is_dark_window,
    LINE_COLOR_DARK,
    LINE_COLOR_LIGHT,
)

from .resources.ui_sync_event_widget import Ui_SyncEventWidget
from .resources.ui_sync_issues_window import Ui_SyncIssuesWindow


# noinspection PyArgumentList
class SyncEventWidget(QtWidgets.QWidget, Ui_SyncEventWidget):
    """
    A widget to graphically display a Maestral sync event.
    """

    def __init__(self, sync_event, parent=None):
        super().__init__(parent=parent)
        self.setupUi(self)

        self.sync_event = sync_event

        self.update_dark_mode()  # set appropriate item icon and colors in style sheet
        self.filenameLabel.setFont(get_scaled_font(0.9))
        self.infoLabel.setFont(get_scaled_font(0.9))

        dirname, filename = osp.split(self.sync_event.local_path)
        parent_dir = osp.basename(dirname)
        change_type = self.sync_event.change_type.value.capitalize()

        dt = datetime.fromtimestamp(self.sync_event.change_time_or_sync_time)
        change_time = dt.strftime("%d %b %Y %H:%M")

        self.filenameLabel.setText(sanitize_string(filename))
        self.infoLabel.setText(f"{change_type} {change_time} • {parent_dir}")

        def request_context_menu():
            self.actionButton.customContextMenuRequested.emit(self.actionButton.pos())

        self.actionButton.setContextMenuPolicy(
            QtCore.Qt.ContextMenuPolicy.CustomContextMenu
        )
        self.actionButton.pressed.connect(request_context_menu)
        self.actionButton.customContextMenuRequested.connect(self.showContextMenu)

    def showContextMenu(self, pos):
        self.actionButtonContextMenu = QtWidgets.QMenu()
        a0 = self.actionButtonContextMenu.addAction("View in folder")
        a1 = self.actionButtonContextMenu.addAction("View on dropbox.com")

        exists = osp.exists(self.sync_event.local_path)
        a0.setEnabled(exists)
        a1.setEnabled(exists)

        a0.triggered.connect(self._go_to_local_path)
        a1.triggered.connect(self._go_to_online)
        self.actionButtonContextMenu.exec(self.mapToGlobal(pos))

    def _go_to_local_path(self):
        click.launch(self.sync_event.local_path, locate=True)

    def _go_to_online(self):
        dbx_address = "https://www.dropbox.com/preview"
        file_address = parse.quote(self.sync_event.dbx_path)
        click.launch(dbx_address + file_address)

    def changeEvent(self, QEvent):
        if QEvent.type() == QtCore.QEvent.Type.ApplicationPaletteChange:
            self.update_dark_mode()

    def update_dark_mode(self):
        # update style sheet with new colors
        line_rgb = LINE_COLOR_DARK if is_dark_window() else LINE_COLOR_LIGHT
        bg_color = self.palette().color(QtGui.QPalette.ColorRole.Base)
        bg_color_rgb = [bg_color.red(), bg_color.green(), bg_color.blue()]
        self.frame.setStyleSheet(
            """
        .QFrame {{
            border: 1px solid rgb({0},{1},{2});
            background-color: rgb({3},{4},{5});
            border-radius: 7px;
        }}""".format(
                *line_rgb, *bg_color_rgb
            )
        )

        # update item icons (the system may supply different icons in dark mode)
        if self.sync_event.item_type is ItemType.File:
            icon = native_item_icon(self.sync_event.local_path)
        else:
            icon = native_folder_icon()
        pixmap = icon_to_pixmap(icon, self.iconLabel.width(), self.iconLabel.height())
        self.iconLabel.setPixmap(pixmap)


# noinspection PyArgumentList
class ActivityWindow(QtWidgets.QWidget, Ui_SyncIssuesWindow):
    """
    A widget to graphically display all Maestral sync history.
    """

    def __init__(self, mdbx, parent=None):
        super().__init__(parent=parent)
        self.setupUi(self)
        self.setWindowFlags(QtCore.Qt.WindowType.WindowStaysOnTopHint)
        self.setWindowTitle("Maestral Activity")

        self.mdbx = mdbx
        self._ids = set()

        self.refresh_gui()

        center_window(self)

        self.update_timer = QtCore.QTimer()
        self.update_timer.timeout.connect(self.refresh_gui)
        self.update_timer.start(1000)  # every 1 sec

    def refresh_gui(self):
        for event in self.mdbx.get_history():
            if event.id not in self._ids:
                event_widget = SyncEventWidget(event)
                self.verticalLayout.insertWidget(0, event_widget)
                self._ids.add(event.id)

    def show(self):
        self.update_timer.start()
        return super().show()

    def close(self):
        self.update_timer.stop()
        return super().close()



# Form implementation generated from reading ui file '/Users/samschott/Python/maestral-qt/maestral_qt/resources/sync_event_widget.ui'
#
# Created by: PyQt6 UI code generator 6.1.1
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets


class Ui_SyncEventWidget(object):
    def setupUi(self, SyncEventWidget):
        SyncEventWidget.setObjectName("SyncEventWidget")
        SyncEventWidget.resize(400, 62)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Maximum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(SyncEventWidget.sizePolicy().hasHeightForWidth())
        SyncEventWidget.setSizePolicy(sizePolicy)
        SyncEventWidget.setMinimumSize(QtCore.QSize(400, 0))
        self.verticalLayout = QtWidgets.QVBoxLayout(SyncEventWidget)
        self.verticalLayout.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout.setObjectName("verticalLayout")
        self.frame = QtWidgets.QFrame(SyncEventWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Maximum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.frame.sizePolicy().hasHeightForWidth())
        self.frame.setSizePolicy(sizePolicy)
        self.frame.setStyleSheet(".QFrame {\n"
"border: 1px solid rgb(205, 203, 205);\n"
"background-color: rgb(255, 255, 255);\n"
"border-radius: 7px;\n"
"}")
        self.frame.setFrameShape(QtWidgets.QFrame.Shape.Box)
        self.frame.setFrameShadow(QtWidgets.QFrame.Shadow.Plain)
        self.frame.setObjectName("frame")
        self.gridLayout = QtWidgets.QGridLayout(self.frame)
        self.gridLayout.setContentsMargins(7, 10, 12, 10)
        self.gridLayout.setObjectName("gridLayout")
        self.infoLabel = QtWidgets.QLabel(self.frame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Preferred, QtWidgets.QSizePolicy.Policy.MinimumExpanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.infoLabel.sizePolicy().hasHeightForWidth())
        self.infoLabel.setSizePolicy(sizePolicy)
        palette = QtGui.QPalette()
        brush = QtGui.QBrush(QtGui.QColor(109, 109, 109))
        brush.setStyle(QtCore.Qt.BrushStyle.SolidPattern)
        palette.setBrush(QtGui.QPalette.ColorGroup.Active, QtGui.QPalette.ColorRole.WindowText, brush)
        brush = QtGui.QBrush(QtGui.QColor(109, 109, 109))
        brush.setStyle(QtCore.Qt.BrushStyle.SolidPattern)
        palette.setBrush(QtGui.QPalette.ColorGroup.Inactive, QtGui.QPalette.ColorRole.WindowText, brush)
        brush = QtGui.QBrush(QtGui.QColor(127, 127, 127))
        brush.setStyle(QtCore.Qt.BrushStyle.SolidPattern)
        palette.setBrush(QtGui.QPalette.ColorGroup.Disabled, QtGui.QPalette.ColorRole.WindowText, brush)
        self.infoLabel.setPalette(palette)
        self.infoLabel.setObjectName("infoLabel")
        self.gridLayout.addWidget(self.infoLabel, 1, 1, 1, 1)
        self.filenameLabel = QElidedLabel(self.frame)
        self.filenameLabel.setObjectName("filenameLabel")
        self.gridLayout.addWidget(self.filenameLabel, 0, 1, 1, 1)
        self.iconLabel = QtWidgets.QLabel(self.frame)
        self.iconLabel.setMinimumSize(QtCore.QSize(32, 32))
        self.iconLabel.setMaximumSize(QtCore.QSize(32, 32))
        self.iconLabel.setText("")
        self.iconLabel.setObjectName("iconLabel")
        self.gridLayout.addWidget(self.iconLabel, 0, 0, 2, 1, QtCore.Qt.AlignmentFlag.AlignHCenter)
        self.actionButton = QtWidgets.QPushButton(self.frame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Fixed, QtWidgets.QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(30)
        sizePolicy.setVerticalStretch(30)
        sizePolicy.setHeightForWidth(self.actionButton.sizePolicy().hasHeightForWidth())
        self.actionButton.setSizePolicy(sizePolicy)
        self.actionButton.setMinimumSize(QtCore.QSize(30, 30))
        self.actionButton.setStyleSheet("QPushButton {\n"
"    border: none;\n"
"    background-color: none;\n"
"    color: rgb(68,133,243);\n"
"    font: bold\n"
"}")
        self.actionButton.setObjectName("actionButton")
        self.gridLayout.addWidget(self.actionButton, 0, 3, 2, 1)
        spacerItem = QtWidgets.QSpacerItem(0, 0, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.gridLayout.addItem(spacerItem, 0, 2, 2, 1)
        self.verticalLayout.addWidget(self.frame)

        self.retranslateUi(SyncEventWidget)
        QtCore.QMetaObject.connectSlotsByName(SyncEventWidget)

    def retranslateUi(self, SyncEventWidget):
        _translate = QtCore.QCoreApplication.translate
        SyncEventWidget.setWindowTitle(_translate("SyncEventWidget", "Form"))
        self.infoLabel.setText(_translate("SyncEventWidget", "Added date * parent folder"))
        self.filenameLabel.setText(_translate("SyncEventWidget", "Metamorphosis.txt"))
        self.actionButton.setText(_translate("SyncEventWidget", "•••"))
from maestral_qt.widgets import QElidedLabel
